/*
 * $Id: skifu.c,v 1.1 2005/02/06 18:43:20 rubini Exp $
 *
 *  Copyright (c) 2003  Daniele Bellucci <belch@linux.it>
 *
 *  skifu:  Simple Kernel Input From Userspace
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/input.h>
#include <asm/uaccess.h>

static char *devname = "skifu";
static struct input_dev skifu_input_dev;	


static ssize_t skifu_write(struct file *file, const char *buf,
			   size_t count, loff_t *ppos)
{
	
	/* report press of key "l" */
	input_report_key(&skifu_input_dev, KEY_L, 1);

	/* report release of key "l" */
	input_report_key(&skifu_input_dev, KEY_L, 0);


	
	/* report press of key "s" */
	input_report_key(&skifu_input_dev, KEY_S, 1);	
	/* report release of key "s" */
	input_report_key(&skifu_input_dev, KEY_S, 0);	


	
	/* report press of key "enter" */
	input_report_key(&skifu_input_dev, KEY_KPENTER, 1);

	/* report release of key "enter" */
	input_report_key(&skifu_input_dev, KEY_KPENTER, 0);

	return count;
}


static struct file_operations proc_skifu_operations = {
	.write		= skifu_write,
};


static int __init skifu_init(void)
{
	struct proc_dir_entry *entry;

	/* create a proc entry to get event from user_space */
	entry = create_proc_entry("skifu-input", S_IWUSR, NULL);

	if (!entry) {
		return -ENOMEM;
	}

	entry->proc_fops = &proc_skifu_operations;

	skifu_input_dev.name = devname;

	/* register a input_dev for KEY_L 
	 *                          KEY_S 
	 *                          KEY_PENTER
	 */

	/* set the event type */
	skifu_input_dev.evbit[0] = BIT(EV_KEY);

	/* set the event codes */
	set_bit(KEY_L, skifu_input_dev.keybit);
	set_bit(KEY_S, skifu_input_dev.keybit);
	set_bit(KEY_KPENTER, skifu_input_dev.keybit);	

	input_register_device(&skifu_input_dev);
	return 0;
}


static void __exit skifu_exit(void)
{
	remove_proc_entry("skifu-input", NULL);
	input_unregister_device(&skifu_input_dev);
}


module_init(skifu_init);
module_exit(skifu_exit);


MODULE_AUTHOR("Daniele Bellucci <belch@linux.it>");
MODULE_DESCRIPTION("skifu: Simple Kernel Input From User Space");
MODULE_LICENSE("GPL");
